"""Module to access Jira API."""

import json
import logging

import requests


class JiraApi:
    """Jira API access."""

    JIRA_BASE_ISSUE_URL = "https://motocommerce.atlassian.net/rest/api/2/issue"

    def __init__(self, jira_email, jira_api_key, logger=None):
        self.jira_email = jira_email
        self.jira_api_key = jira_api_key
        if logger is None:
            self.logger = logging.getLogger()
        else:
            self.logger = logger

    @classmethod
    def get_jira_ticket_key_from_branch(cls, branch):
        """Construct a Jira ticket key based on the git branch name.

        For now it only replaces underscores with dashes and makes it uppercase.
        Eg. "proj_xxxx" -> "PROJ-XXXX"
        """
        if branch:
            jira_ticket_key = branch.upper().replace('_', '-')
        else:
            jira_ticket_key = None
        return jira_ticket_key

    def get_jira_ticket_summary_url(self, jira_ticket_key):
        """Return Jira API endpoint to fetch ticket summary."""
        return "{jira_issue_base_url}/{jira_ticket_key}?fields=summary".format(
            jira_issue_base_url=self.JIRA_BASE_ISSUE_URL,
            jira_ticket_key=jira_ticket_key)

    def get_pr_title_from_jira_ticket(self, branch):
        """Create a pull request title.

        Returns "jira ticket key: summary" as the title. Summary is pulled from
        the Jira ticket (summary field) and Jira ticket is chosen based on the
        branch.
        """
        jira_ticket_key = self.get_jira_ticket_key_from_branch(branch)
        if jira_ticket_key is None:
            error_msg = "Invalid branch name {branch}.".format(branch=branch)
            self.logger.error(error_msg)
            return ""

        url = self.get_jira_ticket_summary_url(jira_ticket_key)
        auth = requests.auth.HTTPBasicAuth(self.jira_email, self.jira_api_key)
        headers = {"Content-Type" : "application/json"}
        try:
            response = requests.get(url=url, auth=auth, headers=headers)
            if response.ok:
                response_data = json.loads(response.text)
                summary = response_data.get("fields", {}).get("summary", "")
                title = "{jira_ticket_key}: {summary}".format(
                    jira_ticket_key=jira_ticket_key, summary=summary)
            else:
                title = jira_ticket_key
        except requests.exceptions.RequestException as ex:
            self.logger.error(ex)
            title = jira_ticket_key
        return title



"""Loading Configuration tests."""

import os

from lazypr.config import get_default_config_file_path


class TestConfig:
    """Test loading configuration."""

    def test_get_default_config_file_path(self):
        """Test getting default config file path."""
        real_home = os.environ.get("HOME")

        os.environ["HOME"] = "/some/test/value"
        expected_path = "/some/test/value/.config/lazy_pr.ini"
        assert get_default_config_file_path() == expected_path

        if real_home is not None:
            os.environ["HOME"] = real_home

    def test_get_default_config_file_path_xdg(self):
        """Test getting config file path when `XDG_CONFIG_HOME` is set."""
        real_config_home = os.environ.get("XDG_CONFIG_HOME")

        os.environ["XDG_CONFIG_HOME"] = "/some/test/value"
        expected_path = "/some/test/value/lazy_pr.ini"
        assert get_default_config_file_path() == expected_path

        if real_config_home is not None:
            os.environ["XDG_CONFIG_HOME"] = real_config_home



"""Create pull request on Github."""

import sys

from pygit2 import GitError, Repository

from .config import load_config
from .github_api import GitHubApi
from .jira_api import JiraApi
from .utils import read_file, setup_logger


def get_branch(repo_local_path=None):
    """Return current branch in a local repository `repo_local_path`."""
    if repo_local_path is None:
        repo_local_path = "."
    try:
        branch = Repository(repo_local_path).head.shorthand
    except GitError:
        branch = None
    return branch


def main():
    """Create pull request on Github."""
    config = load_config()
    logger = setup_logger()

    # Get git branch from config or look for the locally used one in repo-path.
    # If it doesn't exist, ask user to enter the branch via the command line.
    branch = config.get("branch") or get_branch(config.get("repo-path"))
    if branch is None:
        branch = input("Please insert git branch name:\n").strip()
        if branch == "":
            logger.error("Invalid branch.")
            sys.exit(1)

    # Get pull request title from config or create one from the Jira ticket.
    title = config.get("pr-title")
    if title is None:
        jira_api = JiraApi(
            jira_email=config.get("jira-email"),
            jira_api_key=config.get("jira-api-token"),
            logger=logger)
        title = jira_api.get_pr_title_from_jira_ticket(branch=branch)

    # Read pull request description from file (path is set in config).
    description = read_file(path=config.get("pr-desc"))

    # Create pull request.
    gihub_api = GitHubApi(
        github_token=config.get("github-token"), logger=logger)
    repository = gihub_api.get_repository(repository_name=config.get("repo"))
    if repository is None:
        logger.error("No repository found.")
        sys.exit(1)
    gihub_api.create_pull_request(
        repository=repository,
        title=title,
        description=description,
        base=config.get("pr-base", "master"),
        branch=branch,
        team_reviewers=config.get("pr-team"))


if __name__ == "__main__":
    main()



"""Module to access Github API."""

import logging

from github import Github, GithubException, Repository


class GitHubApi:
    """GitHub API access."""

    def __init__(self, github_token, logger=None):
        self.github_token = github_token
        if logger is None:
            self.logger = logging.getLogger()
        else:
            self.logger = logger

    def get_repository(self, repository_name):
        """Fetch and return Github `Repository`."""
        try:
            repository = Github(self.github_token).get_repo(repository_name)
        except GithubException as ex:
            self.logger.error(ex)
            repository = None
        return repository

    def create_pull_request(self, repository, title, description, base, branch,
                            team_reviewers):
        """Create Github `PullRequest` and set team reviewers.

        In case pull request already exists, update title and description.
        """
        assert isinstance(repository, Repository.Repository)

        try:
            pull_request = repository.create_pull(
                title=title, body=description, base=base, head=branch)
            self.set_team_reviewers(pull_request, team_reviewers)
            msg = "Pull request created at {url}".format(
                url=pull_request.html_url)
            self.logger.debug(msg)
        except GithubException:
            pull_request = self.update_pull_request(
                repository=repository, branch=branch, title=title,
                description=description)
            if pull_request:
                msg = "Pull request updated at {url}".format(
                    url=pull_request.html_url)
                self.logger.debug(msg)
        return pull_request

    def update_pull_request(self, repository, branch, title, description):
        """Update Github `PullRequest` - set title and description.

        Find pull request by repository and branch.
        """
        assert isinstance(repository, Repository.Repository)

        pull_requests = repository.get_pulls(
            head="{repo}:{branch}".format(
                repo=repository.full_name, branch=branch),
            state="open")
        try:
            pull_request = pull_requests[0]
            pull_request.edit(body=description, title=title)
        except (IndexError, GithubException) as ex:
            self.logger.error(ex)
            pull_request = None
        return pull_request

    @classmethod
    def set_team_reviewers(cls, pull_request, team_reviewers):
        """Set team reviewers for the given pull request."""
        if not team_reviewers:
            return
        pull_request.create_review_request(team_reviewers=[team_reviewers])



"""Utility functions."""

import logging
import os
import sys


def read_file(path):
    """Read file contents and return it as a string."""
    if path and os.path.exists(path):
        with open(path, "r") as file_handle:
            data = file_handle.read()
    else:
        data = ""
    return data


def setup_logger():
    """Set a logger up with script name logging to STDERR."""
    logger = logging.getLogger(os.path.basename(sys.argv[0]))
    logger.addHandler(logging.StreamHandler(stream=sys.stderr))
    logger.setLevel(logging.DEBUG)
    return logger
